@@ -95,6 +95,16 @@ func NewTargetTester(target *target.Target,
 		injectedSettings: cfgv.NewSettings(nil),
 	}
 
+	if err := t.ensureResolved(false); err != nil {
+		return nil, err
+	}
+
+	if err := t.bspPkg.Reload(t.res.Cfg.SettingValues()); err != nil {
+		return nil, err
+	}
+
+	t.res.Cfg.DetectErrors(bspPkg.FlashMap)
+
 	return t, nil
 }
 
@@ -166,7 +176,7 @@ func (t *TargetBuilder) resolveTransientPkgs(lps []*pkg.LocalPackage) {
 	}
 }
 
-func (t *TargetBuilder) ensureResolved() error {
+func (t *TargetBuilder) ensureResolved(detectErr bool) error {
 	if t.res != nil {
 		return nil
 	}
@@ -230,7 +240,7 @@ func (t *TargetBuilder) ensureResolved() error {
 
 	var err error
 	t.res, err = resolve.ResolveFull(
-		loaderSeeds, appSeeds, t.injectedSettings, t.bspPkg.FlashMap)
+		loaderSeeds, appSeeds, t.injectedSettings, t.bspPkg.FlashMap, detectErr)
 	if err != nil {
 		return err
 	}
@@ -259,7 +269,7 @@ func (t *TargetBuilder) ensureResolved() error {
 }
 
 func (t *TargetBuilder) Resolve() (*resolve.Resolution, error) {
-	if err := t.ensureResolved(); err != nil {
+	if err := t.ensureResolved(true); err != nil {
 		return nil, err
 	}
 
@@ -267,7 +277,7 @@ func (t *TargetBuilder) Resolve() (*resolve.Resolution, error) {
 }
 
 func (t *TargetBuilder) validateAndWriteCfg() error {
-	if err := t.ensureResolved(); err != nil {
+	if err := t.ensureResolved(true); err != nil {
 		return err
 	}
 
@@ -376,7 +386,7 @@ func (t *TargetBuilder) extraADirs() []string {
 }
 
 func (t *TargetBuilder) PrepBuild() error {
-	if err := t.ensureResolved(); err != nil {
+	if err := t.ensureResolved(true); err != nil {
 		return err
 	}
 
@@ -763,29 +773,37 @@ func (t *TargetBuilder) InjectSetting(key string, value string) {
 	t.injectedSettings.Set(key, value)
 }
 
-// Calculates the size of a single boot trailer.  This is the amount of flash
-// that must be reserved at the end of each image slot.
-func (t *TargetBuilder) bootTrailerSize() int {
-	var minWriteSz int
+// Retrieves an integer syscfg flag or returns the default value.
+func getSettingIntValue(settings map[string]syscfg.CfgEntry, settingName string, defaultValue int) int {
 
-	entry, ok := t.res.Cfg.Settings["MCU_FLASH_MIN_WRITE_SIZE"]
+	var intValue int
+	entry, ok := settings[settingName]
 	if !ok {
 		util.StatusMessage(util.VERBOSITY_DEFAULT,
-			"* Warning: target does not define MCU_FLASH_MIN_WRITE_SIZE "+
-				"setting; assuming a value of 1.\n")
-		minWriteSz = 1
+			"* Warning: target does not define %s "+
+				"setting; assuming a value of %d.\n", settingName, defaultValue)
+		intValue = defaultValue
 	} else {
 		val, err := util.AtoiNoOct(entry.Value)
 		if err != nil {
 			util.StatusMessage(util.VERBOSITY_DEFAULT,
 				"* Warning: target specifies invalid non-integer "+
-					"MCU_FLASH_MIN_WRITE_SIZE setting; assuming a "+
-					"value of 1.\n")
-			minWriteSz = 1
+					"%s setting; assuming a "+
+					"value of %d.\n", settingName, defaultValue)
+			intValue = defaultValue
 		} else {
-			minWriteSz = val
+			intValue = val
 		}
 	}
+	return intValue
+}
+
+// Calculates the size of a single boot trailer.  This is the amount of flash
+// that must be reserved at the end of each image slot.
+func (t *TargetBuilder) bootTrailerSize() int {
+
+	minWriteSz := getSettingIntValue(t.res.Cfg.Settings, "MCU_FLASH_MIN_WRITE_SIZE", 1)
+	maxImgSectors := getSettingIntValue(t.res.Cfg.Settings, "BOOTUTIL_MAX_IMG_SECTORS", 128)
 
 	/* Mynewt boot trailer format:
 	 *
@@ -805,11 +823,11 @@ func (t *TargetBuilder) bootTrailerSize() int {
 	 */
 
 	tsize := 16 + // Magic.
-		128*minWriteSz*3 + // Swap status.
+		maxImgSectors*minWriteSz*3 + // Swap status.
 		minWriteSz + // Copy done.
 		minWriteSz // Image Ok.
 
-	log.Debugf("Min-write-size=%d; boot-trailer-size=%d", minWriteSz, tsize)
+	log.Debugf("Min-write-size=%d; max-img-sectors=%d; boot-trailer-size=%d", minWriteSz, maxImgSectors, tsize)
 
 	return tsize
 }
@@ -828,7 +846,7 @@ func (t *TargetBuilder) MaxImgSizes() []int {
 }
 
 func (t *TargetBuilder) CreateDepGraph() (DepGraph, error) {
-	if err := t.ensureResolved(); err != nil {
+	if err := t.ensureResolved(true); err != nil {
 		return nil, err
 	}
 
@@ -836,7 +854,7 @@ func (t *TargetBuilder) CreateDepGraph() (DepGraph, error) {
 }
 
 func (t *TargetBuilder) CreateRevdepGraph() (DepGraph, error) {
-	if err := t.ensureResolved(); err != nil {
+	if err := t.ensureResolved(true); err != nil {
 		return nil, err
 	}
 
