@@ -88,6 +88,12 @@ type Downloader interface {
 	// the repo is in a "detached head" state.
 	CurrentBranch(path string) (string, error)
 
+	// Retrieves commit SHA for current HEAD
+	CurrentHead() string
+
+	// Retrieves full SHA for given commit
+	CommitSha(path string, commit string) (string, error)
+
 	// LatestRc finds the commit of the latest release candidate.  It looks
 	// for commits with names matching the base commit string, but with with
 	// "_rc#" inserted.  This is useful when a release candidate is being
@@ -97,9 +103,19 @@ type Downloader interface {
 	// If such a commit exists, it is returned.  Otherwise, "" is returned.
 	LatestRc(path string, base string) (string, error)
 
+	// Applies patches provided inside "patches" directory.
+	// If no patch is provided function does nothing
+	ApplyPatches(path string, patches []string) error
+
 	// Returns the branch that contains the YAML control files; this option
 	// allows implementers to override "master" as the main branch.
 	MainBranch() string
+
+	// Returns formatted string with repo details
+	String() string
+
+	// Returns if repository was already fetched
+	IsFetched() bool
 }
 
 type Commit struct {
@@ -451,6 +467,26 @@ func (gd *GenericDownloader) Checkout(repoDir string, commit string) error {
 	return err
 }
 
+func (gd *GenericDownloader) ApplyPatches(repoDir string, patches []string) error {
+	cmd := []string{
+		"am",
+	}
+	cmd = append(cmd, patches...)
+
+	_, err := executeGitCommand(repoDir, cmd, true)
+	if err != nil {
+		// Abort git am if applying patches failed
+		cmd = []string{
+			"am",
+			"--abort",
+		}
+		executeGitCommand(repoDir, cmd, true)
+
+		return err
+	}
+	return nil
+}
+
 // Update one submodule tree in a repo (under path)
 func (gd *GenericDownloader) UpdateSubmodule(path string, submodule string) error {
 	cmd := []string{
@@ -564,6 +600,13 @@ func (gd *GenericDownloader) HashFor(path string,
 		return c.hash, nil
 	}
 
+	if len(commit) < 40 {
+		sha, err := gd.CommitSha(path, commit)
+		if err == nil {
+			commit = sha
+		}
+	}
+
 	return commit, nil
 }
 
@@ -640,6 +683,20 @@ func (gd *GenericDownloader) CurrentBranch(path string) (string, error) {
 	return branch, nil
 }
 
+func (gd *GenericDownloader) CurrentHead() string {
+	return gd.head
+}
+
+func (gd *GenericDownloader) CommitSha(path string, commit string) (string, error) {
+	cmd := []string{"rev-parse", commit}
+	o, err := executeGitCommand(path, cmd, true)
+	if err != nil {
+		return "", err
+	}
+
+	return strings.TrimSpace(string(o)), nil
+}
+
 // Fetches the downloader's origin remote if it hasn't been fetched yet during
 // this run.
 func (gd *GenericDownloader) cachedFetch(fn func() error) error {
@@ -763,6 +820,10 @@ func (gd *GenericDownloader) LatestRc(path string,
 	return bestStr, nil
 }
 
+func (gd *GenericDownloader) IsFetched() bool {
+	return gd.fetched
+}
+
 func (gd *GithubDownloader) Fetch(repoDir string) error {
 	return gd.cachedFetch(func() error {
 		util.StatusMessage(util.VERBOSITY_VERBOSE, "Fetching repo %s\n",
@@ -870,11 +931,7 @@ func (gd *GithubDownloader) setRemoteAuth(path string) error {
 func (gd *GithubDownloader) Clone(commit string, dstPath string) error {
 	branch := gd.MainBranch()
 
-	url, publicUrl := gd.remoteUrls()
-
-	util.StatusMessage(util.VERBOSITY_DEFAULT,
-		"Downloading repository %s (commit: %s) from %s\n",
-		gd.Repo, commit, publicUrl)
+	url, _ := gd.remoteUrls()
 
 	gp, err := gitPath()
 	if err != nil {
@@ -890,7 +947,7 @@ func (gd *GithubDownloader) Clone(commit string, dstPath string) error {
 	}
 
 	if util.ShallowCloneDepth > 0 {
-		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth), "--no-single-branch")
 	}
 
 	cmd = append(cmd, url, dstPath)
@@ -909,6 +966,8 @@ func (gd *GithubDownloader) Clone(commit string, dstPath string) error {
 		return err
 	}
 
+	gd.fetched = true
+
 	return nil
 }
 
@@ -936,6 +995,12 @@ func (gd *GithubDownloader) MainBranch() string {
 	}
 }
 
+func (gd *GithubDownloader) String() string {
+	_, publicUrl := gd.remoteUrls()
+
+	return publicUrl
+}
+
 func NewGithubDownloader() *GithubDownloader {
 	return &GithubDownloader{}
 }
@@ -973,9 +1038,6 @@ func (gd *GitDownloader) FetchFile(
 func (gd *GitDownloader) Clone(commit string, dstPath string) error {
 	branch := gd.MainBranch()
 
-	util.StatusMessage(util.VERBOSITY_DEFAULT,
-		"Downloading repository %s (commit: %s)\n", gd.Url, commit)
-
 	gp, err := gitPath()
 	if err != nil {
 		return err
@@ -990,7 +1052,7 @@ func (gd *GitDownloader) Clone(commit string, dstPath string) error {
 	}
 
 	if util.ShallowCloneDepth > 0 {
-		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth))
+		cmd = append(cmd, "--depth", strconv.Itoa(util.ShallowCloneDepth), "--no-single-branch")
 	}
 
 	cmd = append(cmd, gd.Url, dstPath)
@@ -1008,6 +1070,8 @@ func (gd *GitDownloader) Clone(commit string, dstPath string) error {
 		return err
 	}
 
+	gd.fetched = true
+
 	return nil
 }
 
@@ -1033,6 +1097,10 @@ func (gd *GitDownloader) MainBranch() string {
 	}
 }
 
+func (gd *GitDownloader) String() string {
+	return gd.Url
+}
+
 func NewGitDownloader() *GitDownloader {
 	return &GitDownloader{}
 }
@@ -1067,9 +1135,6 @@ func (ld *LocalDownloader) Checkout(path string, commit string) error {
 }
 
 func (ld *LocalDownloader) Clone(commit string, dstPath string) error {
-	util.StatusMessage(util.VERBOSITY_DEFAULT,
-		"Downloading local repository %s\n", ld.Path)
-
 	if err := util.CopyDir(ld.Path, dstPath); err != nil {
 		return err
 	}
@@ -1089,6 +1154,10 @@ func (gd *LocalDownloader) MainBranch() string {
 	return "master"
 }
 
+func (ld *LocalDownloader) String() string {
+	return ld.Path
+}
+
 func NewLocalDownloader() *LocalDownloader {
 	return &LocalDownloader{}
 }
