@@ -199,6 +199,10 @@ func (inst *Installer) ensureDepsInList(repos []*repo.Repo,
 		}
 		for _, d := range deps {
 			depRepo := inst.repos[d.Name]
+			// This might be nil if d.Name is not on allowed repositories list
+			if depRepo == nil {
+				continue
+			}
 			result = append(result, recurse(depRepo)...)
 		}
 
@@ -241,10 +245,6 @@ func (inst *Installer) shouldUpgradeRepo(
 		return true, nil
 	}
 
-	if r.IsUpgradedFromProjectYml() {
-		return false, nil
-	}
-
 	if !r.VersionsEqual(*curVer, destVer) {
 		return true, nil
 	}
@@ -265,6 +265,13 @@ func (inst *Installer) filterUpgradeList(
 	filtered := deprepo.VersionMap{}
 
 	for _, name := range vm.SortedNames() {
+		r := inst.repos[name]
+		if r.IsCheckedForUpgrade() {
+			continue
+		}
+
+		inst.repos[name].SetCheckedForUpgrade()
+
 		ver := vm[name]
 		doUpgrade, err := inst.shouldUpgradeRepo(name, ver)
 		if err != nil {
@@ -282,8 +289,7 @@ func (inst *Installer) filterUpgradeList(
 			}
 			curVer.Commit = ver.Commit
 			util.StatusMessage(util.VERBOSITY_DEFAULT,
-				"Skipping \"%s\": already upgraded (%s)\n",
-				name, curVer.String())
+				"Skipping %s, already upgraded to version %s\n", name, curVer.String())
 		}
 	}
 
@@ -341,8 +347,11 @@ func (inst *Installer) installPrompt(vm deprepo.VersionMap, op installOp,
 		return true, nil
 	}
 
+	if !ask {
+		return true, nil
+	}
 	util.StatusMessage(util.VERBOSITY_DEFAULT,
-		"Making the following changes to the project:\n")
+		"Trying to make the following changes to the project:\n")
 
 	names := vm.SortedNames()
 	for _, name := range names {
@@ -367,10 +376,6 @@ func (inst *Installer) installPrompt(vm deprepo.VersionMap, op installOp,
 		}
 	}
 
-	if !ask {
-		return true, nil
-	}
-
 	for {
 		fmt.Printf("Proceed? [Y/n] ")
 		line, more, err := bufio.NewReader(os.Stdin).ReadLine()
@@ -488,46 +493,6 @@ func (inst *Installer) calcVersionMap(candidates []*repo.Repo) (
 	return vm, nil
 }
 
-// Checks if any repos in the specified slice are in a dirty state.  If any
-// repos are dirty and `force` is *not* enabled, an error is returned.  If any
-// repos are dirty and `force` is enabled, a warning is displayed.
-func verifyRepoDirtyState(repos []*repo.Repo, force bool) error {
-	// [repo] => dirty-state.
-	var m map[*repo.Repo]string
-
-	// Collect all dirty repos and insert them into m.
-	for _, r := range repos {
-		dirtyState, err := r.DirtyState()
-		if err != nil {
-			return err
-		}
-
-		if dirtyState != "" {
-			if m == nil {
-				m = make(map[*repo.Repo]string)
-				m[r] = dirtyState
-			}
-		}
-	}
-
-	if len(m) > 0 {
-		s := "some repos are in a dirty state:\n"
-		for r, d := range m {
-			s += fmt.Sprintf("    %s: contains %s\n", r.Name(), d)
-		}
-
-		if !force {
-			s += "Specify the `-f` (force) switch to attempt anyway"
-			return util.NewNewtError(s)
-		} else {
-			util.OneTimeWarning("%s", s)
-		}
-	}
-
-	return nil
-
-}
-
 func verifyNewtCompat(repos []*repo.Repo, vm deprepo.VersionMap) error {
 	var errors []string
 
@@ -554,10 +519,6 @@ func verifyNewtCompat(repos []*repo.Repo, vm deprepo.VersionMap) error {
 func (inst *Installer) Upgrade(candidates []*repo.Repo, force bool,
 	ask bool) error {
 
-	if err := verifyRepoDirtyState(candidates, force); err != nil {
-		return err
-	}
-
 	vm, err := inst.calcVersionMap(candidates)
 	if err != nil {
 		return err
@@ -591,15 +552,38 @@ func (inst *Installer) Upgrade(candidates []*repo.Repo, force bool,
 	// Upgrade each repo in the version map.
 	for _, r := range repos {
 		destVer := vm[r.Name()]
+		dirtyState, err := r.DirtyState()
+		if err != nil {
+			return err
+		}
+
+		if dirtyState != "" {
+			if !force {
+				util.StatusMessage(util.VERBOSITY_DEFAULT,
+					"%s is in dirty state, it won't be upgraded\n",
+					r.Name())
+				continue
+			} else {
+				util.OneTimeWarning("forced update of repo in dirty state: %s", r.Name())
+			}
+		}
+
+		util.StatusMessage(util.VERBOSITY_DEFAULT,
+			"Upgrading %s to version %s\n", r.Name(), destVer.String())
+
 		if err := r.Upgrade(destVer); err != nil {
 			return err
 		}
-		if r.IsFromProjectYml() {
-			r.SetIsUpgradedFromProjectYml()
+	}
+
+	for _, r := range candidates {
+		err = r.ApplyPatches()
+		if err != nil {
+			util.StatusMessage(util.VERBOSITY_DEFAULT,
+				"Applying patches in repository %s failed\n", r.Name())
+
+			return err
 		}
-		util.StatusMessage(util.VERBOSITY_DEFAULT,
-			"%s successfully upgraded to version %s\n",
-			r.Name(), destVer.String())
 	}
 
 	return nil
