@@ -99,50 +99,35 @@ type MfgEmitter struct {
 // `.bin` files; image targets use `.img`.
 func targetSrcBinPath(t *target.Target, isBoot bool) string {
 	if isBoot {
-		return builder.AppBinPath(t.Name(), builder.BUILD_NAME_BOOT,
-			t.App().Name())
+		return builder.AppBinPath(t.Name(), builder.BUILD_NAME_APP,
+			t.App().FullName())
 	} else {
 		return builder.AppImgPath(t.Name(), builder.BUILD_NAME_APP,
-			t.App().Name())
+			t.App().FullName())
 	}
 }
 
 // Calculates the source path of a target's `.elf` file.
-func targetSrcElfPath(t *target.Target, isBoot bool) string {
-	if isBoot {
-		return builder.AppElfPath(t.Name(), builder.BUILD_NAME_BOOT, t.App().Name())
-	} else {
-		return builder.AppElfPath(t.Name(), builder.BUILD_NAME_APP, t.App().Name())
-	}
+func targetSrcElfPath(t *target.Target) string {
+	return builder.AppElfPath(t.Name(), builder.BUILD_NAME_APP, t.App().FullName())
 }
 
 // Calculates the source path of a target's manifest file.
-func targetSrcManifestPath(t *target.Target, isBoot bool) string {
-	if isBoot {
-		return builder.ManifestPath(t.Name(), builder.BUILD_NAME_BOOT,
-			t.App().Name())
-	} else {
-		return builder.ManifestPath(t.Name(), builder.BUILD_NAME_APP,
-			t.App().Name())
-	}
+func targetSrcManifestPath(t *target.Target) string {
+	return builder.ManifestPath(t.Name(), builder.BUILD_NAME_APP,
+		t.App().FullName())
 }
 
 func newMfgEmitTarget(bt MfgBuildTarget) (MfgEmitTarget, error) {
-	var build_name string
-	if bt.IsBoot {
-		build_name = builder.BUILD_NAME_BOOT
-	} else {
-		build_name = builder.BUILD_NAME_APP
-	}
 	return MfgEmitTarget{
 		Name:    bt.Target.FullName(),
 		Offset:  bt.Area.Offset + bt.Offset,
 		Size:    bt.Size,
 		IsBoot:  bt.IsBoot,
 		BinPath: targetSrcBinPath(bt.Target, bt.IsBoot),
-		ElfPath: targetSrcElfPath(bt.Target, bt.IsBoot),
+		ElfPath: targetSrcElfPath(bt.Target),
 		ManifestPath: builder.ManifestPath(bt.Target.Name(),
-			build_name, bt.Target.App().Name()),
+			builder.BUILD_NAME_APP, bt.Target.App().FullName()),
 		ExtraManifest: bt.ExtraManifest,
 	}, nil
 }
@@ -182,7 +167,7 @@ func getCompilerFromBsp(bsp *pkg.BspPackage) (*toolchain.Compiler, error) {
 	}
 
 	c, err := toolchain.NewCompiler(compilerPkg.BasePath(), "",
-		target.DEFAULT_BUILD_PROFILE)
+		target.DEFAULT_BUILD_PROFILE, nil)
 	if err != nil {
 		return nil, err
 	}
