@@ -44,6 +44,7 @@ const REPO_DEFAULT_PERMS = 0755
 
 const REPO_FILE_NAME = "repository.yml"
 const REPOS_DIR = "repos"
+const PATCHES_DIR = "patches"
 
 type Repo struct {
 	name       string
@@ -54,6 +55,10 @@ type Repo struct {
 	local      bool
 	ncMap      compat.NewtCompatMap
 
+	// Indicates if repo was checked for potential upgrade. This avoids checking
+	// the same repo multiple times in different phases
+	upgradeChecked bool
+
 	// If repo was added from package (not from project.yml), this variable stores name of this package
 	pkgName string
 
@@ -66,16 +71,12 @@ type Repo struct {
 	// version => commit
 	vers map[newtutil.RepoVersion]string
 
-	// Since we are calling upgrade twice - first time for repos from project.yml
-	// and second time for repos from packages, we need to keep track on status
-	// of each repo. If repo is defined in project.yml we want to upgrade it only
-	// once so the version from project.yml stays valid. These flags are used for
-	// this purpose.
-	isFromProjectYml         bool
-	isUpgradedFromProjectYml bool
-
 	hasSubmodules bool
 	submodules    []string
+
+	// Used with external repos. If package that adds external repository provides patches for it,
+	// the paths to them are going to be stored here.
+	patches []string
 }
 
 type RepoDependency struct {
@@ -124,6 +125,19 @@ func (r *Repo) Downloader() downloader.Downloader {
 	return r.downloader
 }
 
+func (r *Repo) AddPatch(path string) {
+	r.patches = append(r.patches, path)
+}
+
+func (r *Repo) ApplyPatches() error {
+	if len(r.patches) == 0 {
+		return nil
+	}
+
+	err := r.Downloader().ApplyPatches(r.Path(), r.patches)
+	return err
+}
+
 func (repo *Repo) FilteredSearchList(
 	curPath string, searchedMap map[string]struct{}) ([]string, error) {
 
@@ -209,22 +223,6 @@ func (r *Repo) patchesFilePath() string {
 		"/.patches/"
 }
 
-func (r *Repo) IsUpgradedFromProjectYml() bool {
-	return r.isUpgradedFromProjectYml
-}
-
-func (r *Repo) SetIsUpgradedFromProjectYml() {
-	r.isUpgradedFromProjectYml = true
-}
-
-func (r *Repo) IsFromProjectYml() bool {
-	return r.isFromProjectYml
-}
-
-func (r *Repo) SetIsFromProjectYml() {
-	r.isFromProjectYml = true
-}
-
 // Checks for repository.yml file presence in specified repo folder.
 // If there is no such file, the repo in specified folder is external.
 func (r *Repo) IsExternal(dir string) bool {
@@ -262,6 +260,9 @@ func (r *Repo) downloadRepo(commit string) error {
 	}
 	defer os.RemoveAll(tmpdir)
 
+	util.StatusMessage(util.VERBOSITY_DEFAULT,
+		"Cloning %s (%s)\n", r.Name(), dl.String())
+
 	// Download the git repo, returns the git repo, checked out to that commit
 	if err := dl.Clone(commit, tmpdir); err != nil {
 		return util.FmtNewtError("Error downloading repository %s: %s",
@@ -384,7 +385,9 @@ func (r *Repo) UpdateDesc() (bool, error) {
 		return false, nil
 	}
 
-	util.StatusMessage(util.VERBOSITY_VERBOSE, "[%s]:\n", r.Name())
+	if !r.downloader.IsFetched() {
+		util.StatusMessage(util.VERBOSITY_DEFAULT, "Fetching %s\n", r.Name())
+	}
 
 	// Make sure the repo's "origin" remote points to the correct URL.  This is
 	// necessary in case the user changed his `project.yml` file to point to a
@@ -393,14 +396,20 @@ func (r *Repo) UpdateDesc() (bool, error) {
 		return false, err
 	}
 
-	// Download `repository.yml`.
-	if err := r.DownloadDesc(); err != nil {
-		return false, err
-	}
+	if r.IsExternal(r.Path()) {
+		if err := r.downloader.Fetch(r.Path()); err != nil {
+			return false, err
+		}
+	} else {
+		// Download `repository.yml`.
+		if err := r.DownloadDesc(); err != nil {
+			return false, err
+		}
 
-	// Read `repository.yml` and populate this repo object.
-	if err := r.Read(); err != nil {
-		return false, err
+		// Read `repository.yml` and populate this repo object.
+		if err := r.Read(); err != nil {
+			return false, err
+		}
 	}
 
 	r.updated = true
@@ -408,6 +417,18 @@ func (r *Repo) UpdateDesc() (bool, error) {
 	return true, nil
 }
 
+func (r *Repo) IsUpdated() bool {
+	return r.updated
+}
+
+func (r *Repo) SetCheckedForUpgrade() {
+	r.upgradeChecked = true
+}
+
+func (r *Repo) IsCheckedForUpgrade() bool {
+	return r.upgradeChecked
+}
+
 func (r *Repo) EnsureExists() error {
 	// Clone the repo if it doesn't exist.
 	if !r.CheckExists() {
