@@ -22,6 +22,7 @@ package ycfg
 import (
 	"fmt"
 	"mynewt.apache.org/newt/newt/cfgv"
+	"reflect"
 	"strings"
 
 	"github.com/spf13/cast"
@@ -469,7 +470,13 @@ func (yc *YCfg) GetStringMap(
 				Expr:  mapEntry.Expr,
 			}
 
-			// XXX: Report collisions?
+			if _, exists := result[k]; exists {
+				if !reflect.DeepEqual(entry.Value, result[k].Value) && (result[k].Expr != nil) && (entry.Expr != nil) {
+					return nil, fmt.Errorf("setting %s collision - two conditions true:\n[%s, %s]\n"+
+						"Conflicting file: %s",
+						k, entry.Expr.String(), result[k].Expr.String(), yc.name)
+				}
+			}
 			result[k] = entry
 		}
 	}
@@ -607,7 +614,13 @@ func (yc *YCfg) GetStringMapString(key string,
 				Expr:  mapEntry.Expr,
 			}
 
-			// XXX: Report collisions?
+			if _, exists := result[k]; exists {
+				if (entry.Value != result[k].Value) && (result[k].Expr != nil) {
+					return nil, fmt.Errorf("Setting %s collision - two conditions true:\n[%s, %s]\n"+
+						"Conflicting file: %s",
+						k, entry.Expr.String(), result[k].Expr.String(), yc.name)
+				}
+			}
 			result[k] = entry
 		}
 	}
@@ -623,6 +636,9 @@ func (yc *YCfg) GetValStringMapString(key string,
 	settings *cfgv.Settings) (map[string]string, error) {
 
 	entryMap, getErr := yc.GetStringMapString(key, settings)
+	if getErr != nil {
+		return nil, getErr
+	}
 
 	valMap := make(map[string]string, len(entryMap))
 	for k, v := range entryMap {
@@ -631,7 +647,7 @@ func (yc *YCfg) GetValStringMapString(key string,
 		}
 	}
 
-	return valMap, getErr
+	return valMap, nil
 }
 
 // FullName calculates a node's name with the following form:
